#!/bin/bash
#
# ACME Certificate Authority Generation v1.0
# 
# 
PARAM1=$1

usage() {
  echo
  echo "This script will generate all the files necessary to build a certificate chain of trust"
  echo "using a CA, CA Intermediate, Server, and Client certificates. After the bootstrap the other"
  echo "helper scripts will generate new certificates"
  echo 
  echo "Usage: cert_bootstrap  <.cnf file (minus the .cnf)>"
  echo
  echo "Example: cert_bootstrap  org.acme.xyz"
  exit 1
}

echo_block() {
  echo
  echo
  echo "***** ***** ***** *****"
  echo $1
  echo "***** ***** ***** *****"
}

#
# CA generation requires .cnf files
# create CA directory
# create bash variables to CA
# restore script back to original path
#
app_init() {
  if [[ -n $PARAM1 ]]; then
    # need to know the location of the configuration file  (expected to be in same dir path as this script)
    CA_CNF="$CD/ca.cnf"

    # handle the case of having the ".cnf" extension or not
    if [[ ${PARAM1: -4} == .cnf ]]; then
      ORG_URL=${PARAM1%.*}
      S_CNF=${PARAM1}
      echo "ASDF: ${ORG_URL},  ${S_CNF}"
    else
      ORG_URL=$PARAM1
      S_CNF="${PARAM1}.cnf"
      echo "ZXCV: ${ORG_URL},  ${S_CNF}"
    fi

    FQ_S_CNF="${CD}/${S_CNF}"
    if [[ ! -f $FQ_S_CNF ]] || [[ ! -f $CA_CNF ]]; then
      usage
    fi
  else
    usage
  fi

  # Organize
  # 
  # create a unique path for the server certificate
  UNIQ_DIR=`date +%Y-%m-%d.%H_%M_%S`
  UNIQ_DIR="cert-chain_${UNIQ_DIR}"
  mkdir -p "${UNIQ_DIR}"
  cd "${UNIQ_DIR}"
  # FQ_DIR="${CD}/${UNIQ_DIR}"
}

#
# IN: UNIQ_ID_CA, SERIAL
#
one-time-ca() {
  # params
  SERIAL="101"
  UNIQ_ID_CA="${SERIAL}.${ORG_URL}"
  CA_DIR="ca_${UNIQ_ID_CA}"
  mkdir $CA_DIR
  cd $CA_DIR
  generate_ca $UNIQ_ID_CA $SERIAL
  FQ_CA_DIR=`pwd`
  FQ_CA_CERT="${FQ_CA_DIR}/ca_${UNIQ_ID_CA}.crt.pem"
  FQ_CA_KEYS="${FQ_CA_DIR}/ca_${UNIQ_ID_CA}.keys.pem"
  cd ..
}

# ***** ***** ***** *****
# 
# CERTIFICATE AUTHORITY
# 
# ***** ***** ***** *****
# This function will generate a CA Intermediate
# IN: UNIQ_ID_CA, SERIAL
#
generate_ca() {
  # params
  UNIQ_ID_CA=$1
  SERIAL=$2
  # encrypt the key
  #openssl genrsa -aes256 -out ca.keys.pem 4096
  #openssl genrsa -aes256 -password "pass:password" -out ca.keys.pem 4096 

  # key un-protected
  openssl genrsa -out "ca_${UNIQ_ID_CA}.keys.pem" 4096
  # 
  # Create Certificate   (valid for 10 years, after the entire chain of trust expires)
  openssl req -config $CA_CNF -new -x509 -sha256 -days 3650 -extensions v3_ca \
              -subj "/C=OO/O=ACME/CN=root.${UNIQ_ID_CA}" -set_serial ${SERIAL} \
              -key ca_${UNIQ_ID_CA}.keys.pem -out ca_${UNIQ_ID_CA}.crt.pem

  # verify certificate (output to text file for review)
  openssl x509 -noout -text -in ca_${UNIQ_ID_CA}.crt.pem > ca_${UNIQ_ID_CA}_cert.info.txt
}

main() {

}

